@@ -21,9 +21,13 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
+#if !(defined __MINGW32__ || defined _MSC_VER)
 #include <sys/ioctl.h>
+#endif
 #include <sys/stat.h>
+#ifdef __unix__
 #include <unistd.h>
+#endif
 
 #include "access.h"
 #include "config.h"
@@ -72,11 +76,15 @@ find_capabilities (int fd, int *c)
     *c |= CAP1284_ECP | CAP1284_ECPRLE;
   if (m & PARPORT_MODE_DMA)
     *c |= CAP1284_DMA;
+  if (!(m & PARPORT_MODE_TRISTATE))
+    *c &= ~(CAP1284_BYTE | CAP1284_ECPSWE);
 }
 
 static int
-init (struct parport_internal *port, int flags, int *capabilities)
+init (struct parport *pport, int flags, int *capabilities)
 {
+  struct parport_internal *port = pport->priv;
+
   if (flags & ~F1284_EXCL)
     return E1284_NOTAVAIL;
 
@@ -87,10 +95,26 @@ init (struct parport_internal *port, int flags, int *capabilities)
   ((struct ppdev_priv *)port->access_priv)->nonblock = 0;
   ((struct ppdev_priv *)port->access_priv)->current_flags = 0;
   port->fd = open (port->device, O_RDWR | O_NOCTTY);
+
+  /* Retry with udev/devfs naming, if available */
   if (port->fd < 0)
     {
-      free (port->access_priv);
-      return E1284_INIT;
+      if (port->udevice)
+	port->fd = open (port->udevice, O_RDWR | O_NOCTTY);
+
+      if (port->fd < 0)
+	{
+	  free (port->access_priv);
+	  return E1284_INIT;
+	}
+      else
+	{
+	  pport->filename = port->udevice;
+	}
+    }
+  else
+    {
+      pport->filename = port->device;
     }
 
   port->current_mode = M1284_COMPAT;
