@@ -22,10 +22,18 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
+#if !(defined __MINGW32__ || defined _MSC_VER)
 #include <sys/ioctl.h>
+#else
+#include <io.h> /* open(), close() */
+#define O_NOCTTY 0
+#define S_ISDIR(mode) ((mode) & _S_IFDIR ? 1 : 0)
+#endif
 #include <sys/stat.h>
 #include <sys/types.h>
+#ifdef __unix__
 #include <unistd.h>
+#endif
 
 #include "debug.h"
 #include "detect.h"
@@ -42,7 +50,11 @@
 #include <sys/ddi.h>
 #include <sys/sunddi.h>
 #elif defined(HAVE_CYGWIN_NT)
+#ifdef __CYGWIN__
 #include <w32api/windows.h>
+#else
+#include <windows.h>
+#endif
 #endif
 
 
@@ -81,13 +93,28 @@ check_dev_node (const char *type)
   char name[50]; /* only callers are in detect_environment */
   int fd;
   int i;
+#ifdef HAVE_LINUX
+  int is_parport;
+#endif
 
   for (i = 0; i < 8; i++) {
     sprintf (name, "/dev/%s%d", type, i);
     fd = open (name, O_RDONLY | O_NOCTTY);
+
+#ifdef HAVE_LINUX
+    is_parport = !strncmp (type, "parport", 7);
+
+    if ((fd < 0) && is_parport) {
+      /* Try with udev/devfs naming */
+      debugprintf("%s isn't accessible, retrying with udev/devfs naming...\n", name);
+      sprintf (name, "/dev/%ss/%d", type, i);
+      fd = open (name, O_RDONLY | O_NOCTTY);
+    }
+#endif
+
     if (fd >= 0) {
 #ifdef HAVE_LINUX
-      if (!strcmp (type, "parport"))
+      if (is_parport)
 	{
 	  /* Try to claim the device.  This will
 	   * force the low-level port driver to get loaded. */
@@ -99,6 +126,8 @@ check_dev_node (const char *type)
       close (fd);
       debugprintf ("%s is accessible\n", name);
       return 1;
+    } else {
+      debugprintf("%s isn't accessible\n", name);
     }
   }
 
@@ -197,6 +226,7 @@ int
 detect_environment (int forbidden)
 {
 #define FORBIDDEN(bit) (forbidden & bit)
+  int dev_node_parport = 0;
   static int detected = 0;
   if (detected && !forbidden) return 0;
   detected = 1;
@@ -204,7 +234,9 @@ detect_environment (int forbidden)
   capabilities = 0;
 
   /* Find out what access mechanisms there are. */
-  if ((!FORBIDDEN(PPDEV_CAPABLE)) && check_dev_node ("parport"))
+  if (!FORBIDDEN(PPDEV_CAPABLE))
+    dev_node_parport = check_dev_node ("parport");
+  if (dev_node_parport)
     capabilities |= PPDEV_CAPABLE;
   if (!FORBIDDEN (IO_CAPABLE))
     check_io ();
@@ -214,7 +246,8 @@ detect_environment (int forbidden)
     check_lpt ();
 
   /* Find out what kind of /proc structure we have. */
-  check_dev_node ("lp"); /* causes low-level port driver to be loaded */
+  if (!dev_node_parport) /* Don't load lp if we'll use ppdev (claim will fail if F1284_EXCL). */
+    check_dev_node ("lp"); /* causes low-level port driver to be loaded */
   check_proc_type ();
 
   return 0;
